/**
 **	File ......... Edit.cpp
 **	Published ....  2004-05-15
 **	Author ....... grymse@alhem.net
**/
/*
Copyright (C) 2004  Anders Hedstrom

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <SDL.h>
#include "guitypedefs.h"
#include "guicolors.h"
#include <sge.h>
//#include <iotm/Mutex.h>

#include "TTFont.h"
#include "SDLControl.h"
#include "Surface.h"
#include "Static.h"
#include "Edit.h"
#include "Xml.h"

using util::Xml;


namespace gui
{


Edit::Edit(Surface *s,coord_t x,coord_t y,coord_t w,coord_t h,SurfaceHelper *pclHelper)
:Surface(s,x,y,w,h,pclHelper)
,m_line("")
,m_echo(0)
,m_bFocus(false)
{
	SetBgColor(EDIT_DEFAULT_BG_R,EDIT_DEFAULT_BG_G,EDIT_DEFAULT_BG_B);
	SetFgColor(EDIT_DEFAULT_FG_R,EDIT_DEFAULT_FG_G,EDIT_DEFAULT_FG_B);
	SetFrameWidth(EDIT_DEFAULT_FW);
	SetFrameColor(EDIT_DEFAULT_FC_R,EDIT_DEFAULT_FC_G,EDIT_DEFAULT_FC_B);
}


Edit::~Edit()
{
}


void Edit::SetEcho(char c)
{
	m_echo = c;
}


void Edit::Draw()
{
	SDL_Rect *area = GetClientRectPtr();
	Uint32 color;
	TTFont *font = GetTTFont();
	SDL_Color fg;
	SDL_Color bg;

	color = GetFgColor();
	memmove(&fg,&color,4);
	color = GetBgColor();
	memmove(&bg,&color,4);

	SDL_FillRect(m_screen, area, color );
	if (font && font -> GetFont())
	{
		coord_t h = font -> GetH();
		coord_t y = GetH() / 2 + h / 2;

		char *s = new char[m_line.size() + 2];
		if (m_echo)
		{
			int i;
			for (i = 0; i < (int)m_line.size(); i++)
				s[i] = m_echo;
			s[i] = 0;
		}
		else
		{
			strcpy(s,m_line.c_str());
		}
		if (m_bFocus)
		{
			strcat(s,"_");
		}
		sge_tt_textoutf(m_screen, font -> GetFont(),
			10,y,
			fg.r,fg.g,fg.b,
			bg.r,bg.g,bg.b,
			SDL_ALPHA_OPAQUE,
			"%s",s); //m_line.c_str());
		delete s;
	}
}


void Edit::OnKeyDown(SDL_keysym keysym)
{
	char c = (char)keysym.sym;

	if (isprint(c))
	{
		m_line += c;
		SetDirty(true);
	}
	else
	if (keysym.sym == SDLK_BACKSPACE || keysym.sym == SDLK_DELETE)
	{
		if (m_line.size())
		{
			m_line.erase(m_line.end() - 1);
			SetDirty(true);
		}
	}
	else
	if (keysym.sym == SDLK_RETURN)
	{
		Surface *pclParent = GetParent();
		if (pclParent)
		{
			surface_event_t e;
			e.type = GUI_EVENT_EDIT;
			e.id = GetID();
			e.local_id = GetLocalID();
			e.ptr = this;
			pclParent -> OnEvent(&e);
		}
	}
	else
	{
		return;
	}
//	DrawAll();
//	InvalidateEv();
}


void Edit::SetLine(const string &str)
{
	m_line = str;
	SetDirty(true);
//	InvalidateEv();
}


const char *Edit::GetLine()
{
	return m_line.c_str();
}


void Edit::OnFocus()
{
	m_bFocus = true;
	SetDirty(true);
}


void Edit::OnLostFocus()
{
	m_bFocus = false;
	SetDirty(true);
}


void Edit::OnHide()
{
	if (m_bFocus)
	{
//		m_bFocus = false;
//		DrawAll();
		SDL_Event e;
	
		e.type = SDL_USEREVENT;
		e.user.code = SDL_USEREVENT_LOSTFOCUS;
		e.user.data1 = this;
		e.user.data2 = NULL;
		SDL_PushEvent(&e);
	}
}


void Edit::DumpMembers(FILE *fil)
{
	Xml xml(fil);

	Surface::DumpMembers(fil);

	xml.Write("LINE",m_line);
	xml.Write("ECHO",m_echo);
}


} // namespace

//sge_TTF_FontHeight 
//sge_TTF_TextSize
