/**
 **	File ......... MenuItem.cpp
 **	Published ....  2004-05-15
 **	Author ....... grymse@alhem.net
**/
/*
Copyright (C) 2004  Anders Hedstrom

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <SDL.h>
#include "guitypedefs.h"
#include "guicolors.h"
#include <sge.h>

#include "TTFont.h"
#include "Surface.h"
#include "Hotspot.h"
#include "MenuItem.h"
#include "Xml.h"

using util::Xml;


namespace gui
{


MenuItem::MenuItem(Surface *s,const string &txt,TTFont *pclFont,SurfaceHelper *pclHelper)
:Surface(s,pclHelper)
,m_bClickCloses(true)
{
	TTFont *font;
	SDL_Rect rect;

	SetText(txt);
	if (pclFont)
	{
		SetTTFont(pclFont);
	}
	SetClickable(true);
	SetInverted(false);

	font = GetTTFont();
	font -> GetRect(txt, rect);

	SetFrameWidth(0);
	SetW(rect.w + rect.h + 8, false);
	SetH(rect.h, false);
}


void MenuItem::CreateMenuItem()
{
	RecreateSurfaces();
}


MenuItem::~MenuItem()
{
}


void MenuItem::Draw()
{
	SDL_Rect *area = GetClientRectPtr();
	Uint32 color;
	TTFont *font = GetTTFont();
	SDL_Color fg;
	SDL_Color bg;

/*
	if (IsInverted())
	{
		color = GetBgColor();
		memmove(&fg,&color,4);
		color = GetFgColor();
		memmove(&bg,&color,4);
	}
	else
	{
*/
		color = GetFgColor();
		memmove(&fg,&color,4);
		color = GetBgColor();
		memmove(&bg,&color,4);
//	}

	// color 0 - transparent
	SDL_FillRect(m_screen, area, color);

	if (font && font -> GetFont())
	{
		coord_t h = font -> GetH();
		coord_t y = GetH() / 2 + h / 2;

		sge_tt_textoutf(m_screen, font -> GetFont(),
			GetH() + 4,y,
			fg.r,fg.g,fg.b,
			bg.r,bg.g,bg.b,
			SDL_ALPHA_OPAQUE,
			"%s",GetText());
	}
}


void MenuItem::OnMouseIn()
{
	if (IsClickable())
	{
		SetInverted(true);
		SetDirty(true);
	}
}


void MenuItem::OnMouseOut()
{
	if (IsClickable())
	{
		SetInverted(false);
		SetDirty(true);
	}
}


void MenuItem::OnClick(coord_t x,coord_t y)
{
	if (m_bClickCloses)
	{
		Surface *head = GetParent();
		
		while (head -> GetBond())
		{
			head = head -> GetBond();
		}
		head -> CloseAll();
		head -> RemoveAllBonds();
	}
	OnButton();
}


void MenuItem::SetClickCloses(bool b)
{
	m_bClickCloses = b;
}


void MenuItem::DumpMembers(FILE *fil)
{
	Xml xml(fil);

	Surface::DumpMembers(fil);

	xml.Write("CLICKCLOSES",m_bClickCloses);
}


} // namespace
