/**
 **	File ......... ScrollBarV.cpp
 **	Published ....  2004-05-15
 **	Author ....... grymse@alhem.net
**/
/*
Copyright (C) 2004  Anders Hedstrom

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <SDL.h>
#include "guitypedefs.h"
#include "guicolors.h"
#include <sge.h>
//#include <iotm/Mutex.h>
#include <SDL_prim.h>

#include "TTFont.h"
#include "SDLControl.h"
#include "Surface.h"
#include "Static.h"
#include "Button.h"
#include "HotButton.h"
#include "ScrollBarV.h"


namespace gui
{


ScrollBarV::ScrollBarV(Surface *s,coord_t x,coord_t y,coord_t w,coord_t h,SurfaceHelper *sh)
:Surface(s,x,y,w,h,sh)
,m_up(this, w - SBSIZE, 0, BUTTONSTYLE_SBUP)
,m_down(this, w - SBSIZE, h - SBSIZE, BUTTONSTYLE_SBDOWN)
,m_pos(0)
,m_max(100)
,m_step(1)
{
	SetBgColor(128,128,0);
	SetFgColor(255,255,128);
	SetFrameWidth(0);
	SetFrameColor(0,255,128);

	AddChild(&m_up);
	AddChild(&m_down);
}


ScrollBarV::ScrollBarV(Surface *s,SurfaceHelper *sh)
:Surface(s,s -> GetClientRectPtr() -> w - SBSIZE,0,
		SBSIZE,
		s -> GetClientRectPtr() -> h - SBSIZE, /* avoid hotspot */
		sh)
,m_up(this, GetW() - SBSIZE, 0, BUTTONSTYLE_SBUP)
,m_down(this, GetW() - SBSIZE, GetH() - SBSIZE, BUTTONSTYLE_SBDOWN)
,m_pos(0)
,m_max(100)
,m_step(1)
{
	SetBgColor(128,128,0);
	SetFgColor(255,255,128);
	SetFrameWidth(0);
	SetFrameColor(0,255,128);

	AddChild(&m_up);
	AddChild(&m_down);
}


ScrollBarV::~ScrollBarV()
{
}


void ScrollBarV::SetPos(long l)
{
	m_pos = l;
	SetDirty(true);
}


void ScrollBarV::SetMax(long l,long step)
{
	m_max = l;
	m_step = step;
}


long ScrollBarV::GetPos()
{
	return m_pos;
}


void ScrollBarV::OnEvent(surface_event_t *pstEvent)
{
	if (pstEvent -> type == GUI_EVENT_BUTTON)
	{
		if (pstEvent -> local_id == m_down.GetLocalID())
		{
			if (m_pos + m_step < m_max)
			{
				m_pos += m_step;
				SetDirty(true);
			}
		}
		else
		if (pstEvent -> local_id == m_up.GetLocalID())
		{
			if (m_pos - m_step >= 0)
			{
				m_pos -= m_step;
				SetDirty(true);
			}
		}
	}
}


void ScrollBarV::Draw()
{
	SDL_Rect a = GetClientRect();
	long range = a.h - 2 * SBSIZE - SBSIZE / 2;
	long y;
	long h;

	SDL_FillRect(m_screen, &a, GetBgColor());

	y = (m_pos * range) / m_max + SBSIZE;
	h = (m_step * range) / m_max;
	if (h < SBSIZE / 2)
	{
		h = SBSIZE / 2;
	}
	a.y = y;
	a.h = h;
	SDL_FillRect(m_screen, &a, GetFgColor());
	a.x++;
	a.y++;
	a.w -= 2;
	a.h -= 2;
	SDL_FillRect(m_screen, &a, GetBgColor());
}


} // namespace gui


