/**
 **	File ......... Surface.h
 **	Published ....  2004-05-15
 **	Author ....... grymse@alhem.net
**/
/*
Copyright (C) 2004  Anders Hedstrom

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#ifndef _SURFACE_H
#define _SURFACE_H


namespace gui
{
class SDLControl;
class SurfaceHelper;
class Color;
class TTFont;

class Surface
{
public:
	Surface(SDLControl *pclControl,
	        coord_t x,coord_t y,coord_t w,coord_t h,SurfaceHelper * = NULL);
	Surface(Surface *pclParent,
	        coord_t x,coord_t y,coord_t w,coord_t h,SurfaceHelper * = NULL);
	Surface(Surface *pclParent,SurfaceHelper * = NULL);
	virtual ~Surface();

	virtual void Dump(FILE *);
	virtual void DumpMembers(FILE *);
	virtual void SaveDialogs();

	void CreateTemporaryScreen();
	void DrawTemporaryScreen();
	void SetUseTemporaryScreen(bool);
	bool UseTemporaryScreen();
	virtual SDL_Surface *GetTemporaryScreen();

	virtual void AddChild(Surface *);
	void BringToTop(Surface *);
	void BringToTop();
	Surface *GetChildOnTop();
	void Remove(Surface *);
	Surface *FindSurface(coord_t,coord_t,coord_t X = 0,coord_t Y = 0);
	void CalculateAbsolute();
	void RecreateSurfaces();
	void FreeSurface(SDL_Surface *);
	void CalculateClientRect();

	void InvalidateEv();
	void InvalidateRectEv(SDL_Rect &);
	void FillRectEv(SDL_Rect &);
	void ResetFillRectsEv();

	virtual void DrawAll();
	void Blit(coord_t X = 0,coord_t Y = 0,SDL_Surface * = NULL);	// Blit: m_screen -> SDL_Surface
	void BlitAbsolute(SDL_Rect *,SDL_Surface * = NULL);
//	void Update8bitPalette();
	void Move(Surface *rootSurface,coord_t,coord_t);
	void MoveTo(Surface *,coord_t,coord_t,bool bVisible = true);

	void Bond(Surface *);
	void Unbond(Surface *);
private:
	void SetBond(Surface *);
public:
	Surface *GetBond();
	void RemoveAllBonds();
	void CloseAll();
	void GetExtents(SDL_Rect &);

	virtual void OnEvent(surface_event_t *);
	virtual void OnMouseIn();
	virtual void OnMouseOut();
	virtual void OnKeyDown(SDL_keysym);
	virtual void OnKeyUp(SDL_keysym);
	virtual void OnButton();
	virtual void OnClose();
	virtual void OnFocus();
	virtual void OnLostFocus();
	virtual void OnHide();
	virtual void OnShow();
	virtual void OnSize(SDL_Rect &,int xrel,int yrel);
	virtual void OnMouseMove(coord_t x,coord_t y,coord_t xrel,coord_t yrel);
	virtual void OnRightClick(coord_t x,coord_t y);
	virtual void OnClick(coord_t x,coord_t y);
	virtual void OnTimer(long id);

	virtual void SetTimer(long time_ms,long id = 0,bool bRepeating = false);
	virtual void SetParent(Surface *);
	virtual void SetX(coord_t);
	virtual void SetY(coord_t);
	virtual void SetW(coord_t,bool bRealloc = true);
	virtual void SetH(coord_t,bool bRealloc = true);
	virtual void SetAbsoluteX(coord_t);
	virtual void SetAbsoluteY(coord_t);
	//
//	virtual void SetBgColor(color_t);
//	virtual void SetFgColor(color_t);
//	virtual void SetFrameColor(color_t);
	virtual void SetBgColor(Uint8,Uint8,Uint8,Uint8 = SDL_ALPHA_OPAQUE);
	virtual void SetFgColor(Uint8,Uint8,Uint8,Uint8 = SDL_ALPHA_OPAQUE);
	virtual void SetFrameColor(Uint8,Uint8,Uint8,Uint8 = SDL_ALPHA_OPAQUE);
	//
	virtual void SetTTFont(TTFont *);
	virtual void SetFrameWidth(coord_t);
	virtual void SetAlpha(alpha_t,bool bRecurse = true);
	virtual void UpdateAlpha(alpha_t);
	virtual void SetLocalID(unsigned long);
//	virtual void SetName(const string &);
//	virtual void SetStyle(style_t);
	virtual void SetText(const string &);
	virtual void SetUICommand(long);
	// flags
	virtual void SetVisible(bool);
	virtual void SetChildrenVisible(bool);
	virtual void SetInverted(bool);
	virtual void SetTransparent(bool);
	virtual void SetMoveable(bool);
	virtual void SetDirty(bool,bool bInvalidate = true);
	virtual void SetResizeArea(bool);
	virtual void SetAnchorRight(bool);
	virtual void SetAnchorDown(bool);
	virtual void SetMustBeDeleted(bool);
	virtual void SetClickable(bool);

	virtual SDL_Surface *GetScreen();
	virtual Surface *GetParent();
	virtual Surface *GetRoot();
	virtual SDL_Rect *GetClientRectPtr();	// always (0,0,w,h)
	virtual SDL_Rect GetClientRect();
	virtual SDL_Rect *GetAbsoluteRectPtr(); // (X,Y,w,h)
	virtual SDL_Rect GetAbsoluteRect();
	virtual SDLControl *GetSDLControl();
	virtual unsigned long GetID();
	virtual unsigned long GetLocalID();
//	virtual const char *GetName();
	virtual coord_t GetX();
	virtual coord_t GetY();
	virtual coord_t GetW();
	virtual coord_t GetH();
	virtual coord_t GetAbsoluteX();
	virtual coord_t GetAbsoluteY();
//	virtual style_t GetStyle();
	virtual const char *GetText();
	virtual long GetUICommand();
	// flags
	virtual bool IsVisible();
	virtual bool IsInverted();
	virtual bool IsTransparent();
	virtual bool IsMoveable();
	virtual bool IsDirty();
	virtual bool IsResizeArea();
	virtual bool IsAnchorRight();
	virtual bool IsAnchorDown();
	virtual bool MustBeDeleted();
	virtual bool IsClickable();

	//
	color_t GetBgColor();
	void GetBgColor(SDL_Color &);
	color_t GetFgColor();
	void GetFgColor(SDL_Color &);
	color_t GetFrameColor();
	void GetFrameColor(SDL_Color &);
	//
	TTFont *GetTTFont();
	coord_t GetFrameWidth();
	alpha_t GetAlpha();

	virtual void DrawFrame(SDL_Surface *,coord_t,Uint32);

protected:
	virtual void Draw();	// Draw: -> m_screen
	SDL_Surface *CreateSDLSurface(bool);	// creates m_screen

	SDL_Surface *m_screen;		// allocated off-screen
	SDL_Surface *m_temp_screen;
	surfacevector_t m_ChildList;

private:
	static unsigned long m_unique_id;
	unsigned long m_id;
	unsigned long m_local_id;
	Surface *m_pclParent;
	SDLControl *m_pclControl;
	SurfaceHelper *m_pclHelper;
	coord_t  m_x;
	coord_t  m_y;
	coord_t  m_w;
	coord_t  m_h;
	coord_t  m_x_absolute;
	coord_t  m_y_absolute;
	//
//	color_t  m_BgColor;
	SDL_Color m_bgColor;
	bool     m_bBgColorIsSet;
//	color_t  m_FgColor;
	SDL_Color m_fgColor;
	bool     m_bFgColorIsSet;
//	color_t  m_FrameColor;
	SDL_Color m_frameColor;
	bool     m_bFrameColorIsSet;
	//
	TTFont * m_pclTTFont;
	alpha_t  m_nAlpha;
	bool     m_bAlphaIsSet;
	coord_t  m_nFrameWidth;
	bool     m_bFrameWidthIsSet;
	SDL_Rect m_clientRect;
//	string   m_strName;
	SDL_Rect m_absoluteRect;
//	style_t  m_style;
	// flags
	bool     m_bVisible;
	bool     m_bChildrenVisible;
	bool     m_bInverted;
	bool     m_bTransparent;
	bool     m_bMoveable;
	bool     m_bDirty;
	bool     m_bResizeArea;
	bool     m_bAnchorRight;
	bool     m_bAnchorDown;
	bool     m_bMustBeDeleted;
	bool     m_bClickable;
	// end of flags
	bool     m_bUseTemporaryScreen;
	string   m_text;
	bool     m_bRedrawTemporary;
	surfacevector_t m_bonds;
	Surface *m_pclBond;
	long     m_uicommand;
};
} // namespace


#endif // _SURFACE_H
